//Willem Wrote this
//Secretbatcave.co.uk

#include "Packet.h"
#include <Ports.h>
#include <RF12.h>
#include <OneWire.h>

const char 	MASTER 		= 1;	// 0 = reciever, 1 = low power temp mode, 2 = powersucking Power monitor
const char	NUMSENSE	= 1;	// to allow more than one sensor
const char	LIGHT		= 1;	// we have a light meter
const char 	DSPIN 		= 17; 	// on port 4

OneWire  	ds(7);  	// on port 4
Port		powerRead (2); 	// on port 2
Port		light(3);	// on port 3
Port		lightRead(1);	// on port 1
int 		triggered = 0;	//to avoid massive amounts of packets coming if there is a light leak


//include an empty interupt to help the wake up
ISR(WDT_vect)
  {
  Sleepy::watchdogEvent();
};	


void setup(){
	//set nodeid here!, although there is no reason not to change it at anytime
	NODEID = 0x02;
	Serial.begin(9600);
	pinMode(DSPIN, OUTPUT);
	//for light sensor
	

	Serial.print("\nSesor node:");
	Serial.println(NODEID, HEX);
	rf12_initialize(NODEID,RF12_868MHZ);
	if (MASTER){
		Serial.println("Mode = Master");
		//turn off the radio for power saving, as we are on battery power here
		rf12_sleep(0);
		}
	//but not in slave mode
	Serial.println("initialised!");
	packet.count = 0;
}

void loop(){
	if (MASTER == 1){
		//switch on the sensor
		digitalWrite(DSPIN, HIGH);
		packet.sensor1 = checkTemp();
		if (NUMSENSE == 2){
			packet.sensor2 = checkTemp();	
		}
		if (LIGHT){
			packet.sensor2 = checkLight();
		}
		packet.battery = rf12_lowbat();
		checkTemp();
		//turn it off again
		digitalWrite(DSPIN, LOW);

		//wake up radio
		rf12_sleep(-1);
		receivePacket();		
		sendPacket(0, 1, 0);
		Serial.println("send packet");	
		

		//turn radio off
		rf12_sleep(0);
		//go to sleep
		Sleepy::loseSomeTime(65535);

	}else if(MASTER == 2){
		//we are in power monitor mode
		//we don't want to use delay, as we might miss a blip
		for (unsigned long x= 0; x < 600000; x++){
			checkBlip();
		}
		Serial.println("check temp");
		packet.sensor1 = checkTemp();
		receivePacket();
		sendPacket(0,1,0);
		checkTemp();
	}else{
		delay(50);
		//listen you muppet
		if(receivePacket() ==true){
			//print it
			if (packetIn->power == 1){
				Serial.println("p:");
			}else{
				Serial.print("t:");
				Serial.print(packetIn->address, HEX );
				Serial.print(" 1:");
				Serial.println(packetIn->sensor1);
				//now to print the rest of the sensors
				if (packetIn->sensor2 ){
					Serial.print("t:");
					Serial.print(packetIn->address, HEX );
					Serial.print(" 2:");
					Serial.println(packetIn->sensor2);
				}	
				if (packetIn->sensor3){
					Serial.print("t:");
					Serial.print(packetIn->address, HEX );
					Serial.print(" 3:");
					Serial.println(packetIn->sensor3);
				}
				if (packetIn->sensor4){
					Serial.print("t:");
					Serial.print(packetIn->address, HEX );
					Serial.print(" 4:");
					Serial.println(packetIn->sensor4);
				}
				Serial.print("Battery status: ");
				Serial.print(packetIn->battery , DEC);
				Serial.print(" Packet Count: ");
				Serial.println(packetIn->count ,DEC);
			}
		}
	}
}

float checkTemp(){
        float tempReading = 0;
        byte present = 0;
        byte data[12];
        byte addr[8];
	

        if ( !ds.search(addr)) {
        //No more addresses to read
                ds.reset_search();
                digitalWrite(DSPIN, LOW);
                return -1000;
        }

        if ( OneWire::crc8( addr, 7) != addr[7]) {
        //CRC is not valid
                digitalWrite(DSPIN, LOW);
		Serial.println("Address CRC fail");
                return -2000;
        }
        ds.reset();
        ds.select(addr);
        ds.write(0x44);         // start conversion
	if (MASTER == 2){
		//we want to check for power blips whilst we are wating
		for (int x =0; x<70; x++){
			//it takes about 120-200 uSeconds to check for a blip
			checkBlip();
			delay(10);
		}
	}else{
		//we delay in the normal way
	        delay (850);		// takes about 750 ms to complete 
	}
        ds.reset();
        ds.select(addr);
        ds.write(0xBE);         // Read the updated data

        for (byte i = 0; i < 9; i++) {           // we need 9 bytes
                data[i] = ds.read();
        }
        tempReading = ((data[1]<< 8) + data[0])* 0.0625;
        Serial.println(tempReading);
        return tempReading;
}

char checkBlip(){
	int reading = powerRead.anaRead();	
	if (reading > 600){
		light.digiWrite(1);
		if (triggered){
			//we already have been triggered
			packet.power = 0;
		
		}else{
			//set packet
			packet.power = 1;
			triggered = 1;
			receivePacket();
			sendPacket(0, 1, 0);
			//don't want to piss out packets.
			delay (100);
		}
		
	}else{
		light.digiWrite(0);
		packet.power = 0;
		triggered = 0;
	}	
}
float checkLight(){
	//I know its retarded to shove a 10bit fixed point into a float, but lazyness is awesome
	int lightReading = 0;
	//turn on LDR
	lightRead.digiWrite(HIGH);
	delay(1);
	lightReading = lightRead.anaRead();
	//done so turn it off
	lightRead.digiWrite(LOW);
	return float(lightReading);
}
